@@ -1,6 +1,6 @@
 # gpxviewer
 #
-# Copyright (C) 2016-2019 Sergey Salnikov <salsergey@gmail.com>
+# Copyright (C) 2016-2020 Sergey Salnikov <salsergey@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3
@@ -17,7 +17,7 @@
 import sys
 import re
 import webbrowser
-from PyQt5.QtCore import Qt, QCoreApplication, QEvent, QFileInfo, QT_VERSION_STR, PYQT_VERSION_STR, pyqtSlot
+from PyQt5.QtCore import Qt, QCoreApplication, QEvent, QFileInfo, QFileSelector, QT_VERSION_STR, PYQT_VERSION_STR, pyqtSlot
 from PyQt5.QtGui import QCursor, QIcon, QKeyEvent, QKeySequence
 from PyQt5.QtWidgets import (QAction, QApplication, QDialog, QDialogButtonBox, QFileDialog, QHBoxLayout, QLabel, QLineEdit,
                              QMainWindow, QMenu, QMessageBox, QSizePolicy, QSpacerItem, QSpinBox, QVBoxLayout, QWidget)
@@ -36,26 +36,9 @@ class GpxMainWindow(QMainWindow):
     super(GpxMainWindow, self).__init__()
     self.ui = gpxviewer.ui_mainwindow.Ui_MainWindow()
     self.ui.setupUi(self)
+    self.updateTheme()
     self.ui.wptView.setFocus()
 
-    self.setWindowIcon(QIcon(':/icons/gpxviewer.svg'))
-    self.ui.actionLoadGPXfile.setIcon(QIcon(':/icons/xml-element-new.svg'))
-    self.ui.actionSaveGPXfileAs.setIcon(QIcon(':/icons/document-save-as-template.svg'))
-    self.ui.actionNew.setIcon(QIcon(':/icons/document-new.svg'))
-    self.ui.actionOpen.setIcon(QIcon(':/icons/document-open.svg'))
-    self.ui.actionSave.setIcon(QIcon(':/icons/document-save.svg'))
-    self.ui.actionSaveAs.setIcon(QIcon(':/icons/document-save-as.svg'))
-    self.ui.menuRecentProjects.setIcon(QIcon(':/icons/document-open-recent.svg'))
-    self.ui.actionQuit.setIcon(QIcon(':/icons/application-exit.svg'))
-    self.ui.actionCopy.setIcon(QIcon(':/icons/edit-copy.svg'))
-    self.ui.actionSettings.setIcon(QIcon(':/icons/configure.svg'))
-    self.ui.actionDistanceProfile.setIcon(QIcon(':/icons/distanceprofile.svg'))
-    self.ui.actionTimeProfile.setIcon(QIcon(':/icons/timeprofile.svg'))
-    self.ui.actionStatistics.setIcon(QIcon(':/icons/view-statistics.svg'))
-    self.ui.actionGpxViewerHelp.setIcon(QIcon(':/icons/help-contents.svg'))
-    self.ui.actionAboutQt.setIcon(QIcon.fromTheme('qtlogo', QIcon(':/icons/qtlogo.svg')))
-    self.ui.actionAboutGPXViewer.setIcon(QIcon(':/icons/gpxviewer.svg'))
-
     self.ui.actionNew.setShortcut(QKeySequence.New)
     self.ui.actionOpen.setShortcut(QKeySequence.Open)
     self.ui.actionSave.setShortcut(QKeySequence.Save)
@@ -91,6 +74,7 @@ class GpxMainWindow(QMainWindow):
     self.ui.wptView.installEventFilter(self)
     self.ui.trkView.installEventFilter(self)
 
+    self.ui.actionDetailedView.setChecked(TheConfig['MainWindow'].getboolean('DetailedView'))
     self.ui.actionShowSkipped.setChecked(TheConfig['MainWindow'].getboolean('ShowSkipped'))
     self.ui.actionShowMarked.setChecked(TheConfig['MainWindow'].getboolean('ShowMarked'))
     self.ui.actionShowCaptioned.setChecked(TheConfig['MainWindow'].getboolean('ShowCaptioned'))
@@ -112,9 +96,31 @@ class GpxMainWindow(QMainWindow):
     self.initColumnsToCopy()
     self.resize(TheConfig['MainWindow'].getint('WindowWidth'), TheConfig['MainWindow'].getint('WindowHeight'))
 
-    self.plot = plt.PlotWindow()
-    self.plot.profileChanged.connect(self.setProjectChanged)
-    self.stat = stat.StatWindow()
+    self.plotWindow = plt.PlotWindow()
+    self.plotWindow.profileChanged.connect(self.setProjectChanged)
+    self.statWindow = stat.StatWindow()
+
+  def updateTheme(self):
+    self.themeSelector = QFileSelector()
+    self.themeSelector.setExtraSelectors([TheConfig['MainWindow']['ColorTheme']])
+
+    self.setWindowIcon(QIcon(':/icons/gpxviewer.svg'))
+    self.ui.actionLoadGPXfile.setIcon(QIcon(self.themeSelector.select(':/icons/xml-element-new.svg')))
+    self.ui.actionSaveGPXfileAs.setIcon(QIcon(self.themeSelector.select(':/icons/document-save-as-template.svg')))
+    self.ui.actionNew.setIcon(QIcon(self.themeSelector.select(':/icons/document-new.svg')))
+    self.ui.actionOpen.setIcon(QIcon(self.themeSelector.select(':/icons/document-open.svg')))
+    self.ui.actionSave.setIcon(QIcon(self.themeSelector.select(':/icons/document-save.svg')))
+    self.ui.actionSaveAs.setIcon(QIcon(self.themeSelector.select(':/icons/document-save-as.svg')))
+    self.ui.menuRecentProjects.setIcon(QIcon(self.themeSelector.select(':/icons/document-open-recent.svg')))
+    self.ui.actionQuit.setIcon(QIcon(':/icons/application-exit.svg'))
+    self.ui.actionCopy.setIcon(QIcon(self.themeSelector.select(':/icons/edit-copy.svg')))
+    self.ui.actionSettings.setIcon(QIcon(self.themeSelector.select(':/icons/configure.svg')))
+    self.ui.actionDistanceProfile.setIcon(QIcon(self.themeSelector.select(':/icons/distanceprofile.svg')))
+    self.ui.actionTimeProfile.setIcon(QIcon(self.themeSelector.select(':/icons/timeprofile.svg')))
+    self.ui.actionStatistics.setIcon(QIcon(self.themeSelector.select(':/icons/view-statistics.svg')))
+    self.ui.actionGpxViewerHelp.setIcon(QIcon(self.themeSelector.select(':/icons/help-contents.svg')))
+    self.ui.actionAboutQt.setIcon(QIcon.fromTheme('qtlogo', QIcon(':/icons/qtlogo.svg')))
+    self.ui.actionAboutGPXViewer.setIcon(QIcon(':/icons/gpxviewer.svg'))
 
   @pyqtSlot()
   def onAboutQt(self):
@@ -149,61 +155,61 @@ class GpxMainWindow(QMainWindow):
         event.ignore()
         return
 
-    self.plot.close()
-    self.stat.close()
+    self.plotWindow.close()
+    self.statWindow.close()
     TheDocument.close()
     TheConfig.save()
     super(GpxMainWindow, self).closeEvent(event)
 
   def wptContextMenuEvent(self, event):
     if self.ui.wptView.selectionModel().hasSelection():
-      actSkip = QAction(QIcon(':/icons/waypoint-skip.svg'), self.tr('Skip points'), self)
+      actSkip = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-skip.svg')), self.tr('Skip points'), self)
       actSkip.setStatusTip(self.tr('Skip these waypoints when plotting profiles or calculating statistics'))
       actSkip.setCheckable(True)
       actSkip.setChecked(not self.ui.wptView.currentIndex().data(gpx.IncludeRole))
       actSkip.triggered.connect(self.skipPoints)
 
-      actMarker = QAction(QIcon(':/icons/waypoint-marker.svg'), self.tr('Points with markers'), self)
+      actMarker = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-marker.svg')), self.tr('Points with markers'), self)
       actMarker.setStatusTip(self.tr('Add markers to these waypoints when plotting profiles'))
       actMarker.setDisabled(actSkip.isChecked())
       actMarker.setCheckable(True)
       actMarker.setChecked(self.ui.wptView.currentIndex().data(gpx.MarkerRole))
       actMarker.triggered.connect(self.markerPoints)
 
-      actCaption = QAction(QIcon(':/icons/waypoint-caption.svg'), self.tr('Points with captions'), self)
+      actCaption = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-caption.svg')), self.tr('Points with captions'), self)
       actCaption.setStatusTip(self.tr('Add captions to these waypoints when plotting profiles'))
       actCaption.setDisabled(actSkip.isChecked())
       actCaption.setCheckable(True)
       actCaption.setChecked(self.ui.wptView.currentIndex().data(gpx.CaptionRole))
       actCaption.triggered.connect(self.captionPoints)
 
-      actSplit = QAction(QIcon(':/icons/waypoint-splitline.svg'), self.tr('Points with splitting lines'), self)
+      actSplit = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-splitline.svg')), self.tr('Points with splitting lines'), self)
       actSplit.setStatusTip(self.tr('Add splitting lines to these waypoints when plotting profiles'))
       actSplit.setDisabled(actSkip.isChecked())
       actSplit.setCheckable(True)
       actSplit.setChecked(self.ui.wptView.currentIndex().data(gpx.SplitLineRole))
       actSplit.triggered.connect(self.splitLines)
 
-      actNeglect = QAction(QIcon(':/icons/waypoint-neglect-distance.svg'), self.tr('Neglect previous distance'), self)
+      actNeglect = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-neglect-distance.svg')), self.tr('Neglect previous distance'), self)
       actNeglect.setStatusTip(self.tr('Neglect distance before these waypoints when plotting profiles'))
       actNeglect.setDisabled(actSkip.isChecked())
       actNeglect.setCheckable(True)
       actNeglect.setChecked(self.ui.wptView.currentIndex().data(gpx.NeglectRole))
       actNeglect.triggered.connect(self.neglectDistance)
 
-      actReset = QAction(QIcon(':/icons/edit-clear-all.svg'), self.tr('Reset appearance'), self)
+      actReset = QAction(QIcon(self.themeSelector.select(':/icons/edit-clear-all.svg')), self.tr('Reset appearance'), self)
       actReset.setStatusTip(self.tr('Reset appearance of these waypoints'))
       actReset.setDisabled(actSkip.isChecked())
       actReset.triggered.connect(self.resetPoints)
 
-      actRename = QAction(QIcon(':/icons/edit-rename.svg'), self.tr('Rename...'), self)
+      actRename = QAction(QIcon(self.themeSelector.select(':/icons/edit-rename.svg')), self.tr('Rename...'), self)
       actRename.setStatusTip(self.tr('Rename these waypoints'))
       actRename.triggered.connect(self.renamePoints)
-      actResetName = QAction(QIcon(':/icons/edit-clear.svg'), self.tr('Reset name'), self)
+      actResetName = QAction(QIcon(self.themeSelector.select(':/icons/edit-clear.svg')), self.tr('Reset name'), self)
       actResetName.setStatusTip(self.tr('Reset the names of these waypoints'))
       actResetName.triggered.connect(self.resetPointNames)
 
-      actStyle = QAction(QIcon(':/icons/configure.svg'), self.tr('Point style'), self)
+      actStyle = QAction(QIcon(self.themeSelector.select(':/icons/configure.svg')), self.tr('Point style'), self)
       actStyle.setStatusTip(self.tr('Change the style of these waypoints when plotting profiles'))
       actStyle.triggered.connect(self.pointStyle)
 
@@ -242,7 +248,7 @@ class GpxMainWindow(QMainWindow):
       actShowTopoMap.triggered.connect(self.showTopoMap)
 
       showMapMenu = QMenu(self.tr('Show on map'), self)
-      showMapMenu.setIcon(QIcon(':/icons/internet-services.svg'))
+      showMapMenu.setIcon(QIcon(self.themeSelector.select(':/icons/internet-services.svg')))
       showMapMenu.addAction(actShowGoogleMap)
       showMapMenu.addAction(actShowYandexMap)
       showMapMenu.addAction(actShowZoomEarthMap)
@@ -256,7 +262,7 @@ class GpxMainWindow(QMainWindow):
 
   def trkContextMenuEvent(self, event):
     if self.ui.trkView.selectionModel().hasSelection():
-      actSkip = QAction(QIcon(':/icons/waypoint-skip.svg'), self.tr('Skip tracks'), self)
+      actSkip = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-skip.svg')), self.tr('Skip tracks'), self)
       actSkip.setStatusTip(self.tr('Skip these tracks when plotting profiles or calculating statistics'))
       actSkip.setCheckable(True)
       actSkip.setChecked(not self.ui.trkView.currentIndex().data(gpx.IncludeRole))
@@ -303,7 +309,7 @@ class GpxMainWindow(QMainWindow):
                 self.tr('Using') + ' Python ' + str(sys.version_info.major) + '.' + str(sys.version_info.minor) + '.' + str(sys.version_info.micro) + ', ' + \
                 'PyQt5 ' + PYQT_VERSION_STR + ', ' + \
                 'Qt ' + QT_VERSION_STR + '<br><br>' + \
-                'Copyright 2016-2019 Sergey Salnikov <a href=mailto:salsergey@gmail.com>&lt;salsergey@gmail.com&gt;</a><br><br>' + \
+                'Copyright 2016-2020 Sergey Salnikov <a href=mailto:salsergey@gmail.com>&lt;salsergey@gmail.com&gt;</a><br><br>' + \
                 self.tr('License:') + ' <a href=http://www.gnu.org/licenses/gpl.html>GNU General Public License, version 3</a>'
     QMessageBox.about(self, self.tr('About GPX Viewer'), aboutText)
 
@@ -331,6 +337,8 @@ class GpxMainWindow(QMainWindow):
                            <ul>
                            <li>Left/right mouse buttons select markers or captions</li>
                            <li>Selected captions are moved by up/down/left/right buttons</li>
+                           <li>Mouse wheel zooms the image horizontally</li>
+                           <li>Left mouse button drags the image horizontally</li>
                            <li>Mouse wheel over the left axis changes minimum/maximum altitude</li>
                            </ul>''')
     msg = QMessageBox(self)
@@ -392,7 +400,7 @@ class GpxMainWindow(QMainWindow):
 
     dlg = QDialog(self)
     dlg.setWindowTitle(self.tr('Rename waypoints'))
-    vLayout = QVBoxLayout(dlg)
+    vLayout = QVBoxLayout()
     vLayout.addWidget(QLabel(self.tr('Enter new name for ') + str(len(indexes)) + self.tr(' waypoints:')))
     dlg.setLayout(vLayout)
 
@@ -401,7 +409,7 @@ class GpxMainWindow(QMainWindow):
     nameEdit.setSelection(0, len(first_name))
     vLayout.addWidget(nameEdit)
 
-    hLayout = QHBoxLayout(dlg)
+    hLayout = QHBoxLayout()
     hLayout.addWidget(QLabel(self.tr('Several symbols "#" will be replaced by sequential numbers starting from:')))
     vLayout.addLayout(hLayout)
 
@@ -573,6 +581,9 @@ class GpxMainWindow(QMainWindow):
       return False
 
   def openGPXFiles(self, filenames):
+    if len(filenames) == 0:
+      return
+
     for f in filenames:
       self.openGPXFile(f)
     TheDocument.gpxparser.updatePoints()
@@ -679,13 +690,15 @@ class GpxMainWindow(QMainWindow):
       QMessageBox.warning(self, self.tr('Plot error'), self.tr('Not enouph points or tracks.'))
       return
 
-    self.plot.setWindowTitle(self.tr('Distance Profile'))
-    self.plot.show()
+    self.plotWindow.setWindowTitle(self.tr('Distance Profile'))
+    self.plotWindow.show()
     if TheConfig.getValue('ProfileStyle', 'SelectedPointsOnly'):
-      self.plot.plotProfile(gpx.DIST, [i.data(gpx.IDRole) for i in self.ui.wptView.selectionModel().selectedRows()], [i.row() for i in self.ui.trkView.selectionModel().selectedRows()])
+      self.plotWindow.plotProfile(gpx.DIST,
+                                  [i.data(gpx.IDRole) for i in self.ui.wptView.selectionModel().selectedRows()],
+                                  [i.row() for i in self.ui.trkView.selectionModel().selectedRows()])
     else:
-      self.plot.plotProfile(gpx.DIST)
-    self.plot.activateWindow()
+      self.plotWindow.plotProfile(gpx.DIST)
+    self.plotWindow.activateWindow()
 
   @pyqtSlot()
   def onPlotTimeProfile(self):
@@ -698,20 +711,23 @@ class GpxMainWindow(QMainWindow):
         break
     if n < 2:
       for j in range(TheDocument.trkmodel.rowCount()):
-        if TheDocument.trkmodel.index(j, 0).data(gpx.IncludeRole) and TheDocument.trkmodel.index(i, gpx.TRKTIME).data() != '':
+        if TheDocument.trkmodel.index(j, 0).data(gpx.IncludeRole) and TheDocument.trkmodel.index(j, gpx.TRKTIME).data() != '':
           n = 2
           break
       if n < 2:
         QMessageBox.warning(self, self.tr('Plot error'), self.tr('Not enouph points or tracks.'))
         return
 
-    self.plot.setWindowTitle(self.tr('Time Profile'))
-    self.plot.show()
+    self.plotWindow.setWindowTitle(self.tr('Time Profile'))
+    self.plotWindow.show()
+    column = gpx.TIME if TheConfig.getValue('ProfileStyle', 'AbsoluteTime') else gpx.TIME_DAYS
     if TheConfig.getValue('ProfileStyle', 'SelectedPointsOnly'):
-      self.plot.plotProfile(gpx.TIME_DAYS, [i.data(gpx.IDRole) for i in self.ui.wptView.selectionModel().selectedRows()], [i.row() for i in self.ui.trkView.selectionModel().selectedRows()])
+      self.plotWindow.plotProfile(column,
+                                  [i.data(gpx.IDRole) for i in self.ui.wptView.selectionModel().selectedRows()],
+                                  [i.row() for i in self.ui.trkView.selectionModel().selectedRows()])
     else:
-      self.plot.plotProfile(gpx.TIME_DAYS)
-    self.plot.activateWindow()
+      self.plotWindow.plotProfile(column)
+    self.plotWindow.activateWindow()
 
   @pyqtSlot()
   def onShowStatistics(self):
@@ -719,8 +735,14 @@ class GpxMainWindow(QMainWindow):
       QMessageBox.warning(self, self.tr('Statistics error'), self.tr('Not enouph points.'))
       return
 
-    self.stat.show()
-    self.stat.activateWindow()
+    self.statWindow.show()
+    self.statWindow.activateWindow()
+
+  @pyqtSlot(bool)
+  def onDetailedView(self, enable):
+    TheConfig['MainWindow']['DetailedView'] = str(enable)
+    self.filterModel.invalidateFilter()
+    self.initColumnsToCopy()
 
   @pyqtSlot(bool)
   def onShowSkipped(self, show):
@@ -758,7 +780,7 @@ class GpxMainWindow(QMainWindow):
   @pyqtSlot()
   def onShowSettings(self):
     dlg = SettingsDialog(self)
-    if dlg.exec_() == QDialog.Accepted:
+    if dlg.exec_() == QDialog.Accepted and (TheDocument.wptmodel.rowCount() != 0 or TheDocument.trkmodel.rowCount() != 0):
       self.setProjectChanged()
       TheDocument.gpxparser.updateDistance()
       self.ui.wptView.resizeColumnsToContents()
@@ -835,7 +857,11 @@ class GpxMainWindow(QMainWindow):
     self.ui.actionClearList.setDisabled(True)
 
   def initColumnsToCopy(self):
-    for i, f in enumerate(TheDocument.wptmodel.fields):
+    for act in self.actionsColumns:
+      self.ui.menuCoLumns.removeAction(act)
+    self.actionsColumns = []
+
+    for i, f in enumerate(TheDocument.wptmodel.fields[:self.filterModel.columnCount()]):
       act = QAction(f, self)
       act.setCheckable(True)
       act.triggered[bool].connect(self.columnsToCopyChanged)
