@@ -1,6 +1,6 @@
 # gpxviewer
 #
-# Copyright (C) 2016-2019 Sergey Salnikov <salsergey@gmail.com>
+# Copyright (C) 2016-2020 Sergey Salnikov <salsergey@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3
@@ -17,17 +17,16 @@
 import xml.etree.ElementTree as ET
 from datetime import datetime, timedelta
 from math import acos, cos, modf, pi, sin, sqrt
-from PyQt5.QtCore import Qt, QAbstractTableModel, QModelIndex, QObject, QPointF, QSortFilterProxyModel, pyqtSignal
+from PyQt5.QtCore import Qt, QAbstractTableModel, QFileInfo, QModelIndex, QObject, QPointF, QSortFilterProxyModel, pyqtSignal
 from PyQt5.QtGui import QColor, QFont, QGuiApplication, QPainter, QPainterPath, QPen, QPixmap, QPolygonF
 from gpxviewer.configstore import TheConfig
 
 
-WPTFIELDS = NAME, LAT, LON, ALT, DIST, TIME, TIMEDELTA, TIME_DAYS = range(8)
+WPTFIELDS = NAME, LAT, LON, ALT, DIST, TIME, TIME_DELTA, TIME_DAYS, DIST_DELTA, ALT_DELTA, SPEED, ALT_SPEED, SLOPE = range(13)
 TRKFIELDS = TRKNAME, TRKSEGS, TRKPTS, TRKLEN, TRKTIME, TRKDUR = range(6)
 ValueRole, IDRole, IncludeRole, MarkerRole, CaptionRole, SplitLineRole, NeglectRole, MarkerStyleRole, CaptionStyleRole, SplitLineStyleRole = range(Qt.UserRole, Qt.UserRole + 10)
-SkipColor, MarkerColor, CaptionColor = (QColor(255, 225, 225), QColor(225, 225, 255), QColor(225, 255, 225))
-MARKER_COLOR, MARKER_STYLE, MARKER_SIZE, CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE, LINE_COLOR, LINE_STYLE, LINE_WIDTH = \
-  ('MarkerColor', 'MarkerStyle', 'MarkerSize', 'CaptionPositionX', 'CaptionPositionY', 'CaptionSize', 'SplitLineColor', 'SplitLineStyle', 'SplitLineWidth')
+MARKER_COLOR, MARKER_STYLE, MARKER_SIZE, CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE, LINE_COLOR, LINE_STYLE, LINE_WIDTH = \
+  ('MarkerColor', 'MarkerStyle', 'MarkerSize', 'CaptionPositionX', 'CaptionPositionY', 'CaptionRotation', 'CaptionSize', 'SplitLineColor', 'SplitLineStyle', 'SplitLineWidth')
 
 
 class GpxWarning(Exception):
@@ -38,16 +37,24 @@ class GpxWarning(Exception):
 class WptModel(QAbstractTableModel):
   def __init__(self, parent=None):
     super(WptModel, self).__init__(parent)
+
     # Workaround to init columns to copy for the first time
     if len(TheConfig.columnsToCopy) == 0:
       TheConfig.columnsToCopy = list(WPTFIELDS)
 
-    self.fields = [self.tr('Name'), self.tr('Latitude'), self.tr('Longitude'), self.tr('Elevation'),
-                   self.tr('Distance'), self.tr('Time'), self.tr('Time difference'), self.tr('Time in days')]
+    self.fields = [self.tr('Name'), self.tr('Latitude'), self.tr('Longitude'), self.tr('Altitude (m)'),
+                   self.tr('Distance (km)'), self.tr('Time'), self.tr('Time difference'), self.tr('Time in days'),
+                   self.tr('Distance difference (km)'), self.tr('Altitude difference (m)'), self.tr('Speed (km/h)'), self.tr('Climbing speed (m/h)'), self.tr('Slope (m/km)')]
     self.resetModel()
     self.pix = QPixmap(16, 16)
     self.pix.fill(Qt.transparent)
 
+    # Define colors that should look well for any color theme
+    lightness = max(50, min(240, QGuiApplication.palette().base().color().lightness()))
+    self.SkipColor = QColor.fromHsl(0, lightness, lightness)
+    self.MarkerColor = QColor.fromHsl(240, lightness, lightness)
+    self.CaptionColor = QColor.fromHsl(120, lightness, lightness)
+
   def rowCount(self, parent=None):
     if parent is not None and parent.isValid():
       return 0
@@ -66,7 +73,7 @@ class WptModel(QAbstractTableModel):
         return self.changedNames[index.row()]
       elif index.column() == TIME and self.waypoints[index.row()][index.column()] != '':
         return str(self.waypoints[index.row()][index.column()] + timedelta(minutes=TheConfig.getValue('ProfileStyle', 'TimeZoneOffset')))
-      elif index.column() == LAT or index.column() == LON:
+      elif index.column() in {LAT, LON}:
         if TheConfig.getValue('ProfileStyle', 'CoordinateFormat') == 0:  # Decimal degrees
           return str(round(self.waypoints[index.row()][index.column()], 6))
         elif TheConfig.getValue('ProfileStyle', 'CoordinateFormat') == 1:  # Degrees with decimal minutes
@@ -79,6 +86,10 @@ class WptModel(QAbstractTableModel):
           sec, min = modf(min)
           sec = round(sec * 60, 2)
           return str(int(deg)) + '°' + str(int(min)) + '\'' + str(sec) + '"'
+      elif index.column() in {ALT, ALT_DELTA, ALT_SPEED, SLOPE} and self.waypoints[index.row()][index.column()] != '':
+        return str(round(self.waypoints[index.row()][index.column()]))
+      elif index.column() in {DIST, TIME_DAYS, DIST_DELTA, SPEED} and self.waypoints[index.row()][index.column()] != '':
+        return str(round(self.waypoints[index.row()][index.column()], 3))
       else:
         return str(self.waypoints[index.row()][index.column()])
     elif role == Qt.DecorationRole and index.column() == NAME:
@@ -106,10 +117,10 @@ class WptModel(QAbstractTableModel):
       else:
         return {k: TheConfig.getValue('PointStyle', k) for k in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}}
     elif role == CaptionStyleRole:
-      if all([k in self.pointStyles[index.row()] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}]):
-        return {k: self.pointStyles[index.row()][k] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}}
+      if all([k in self.pointStyles[index.row()] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}]):
+        return {k: self.pointStyles[index.row()][k] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}}
       else:
-        return {k: TheConfig.getValue('PointStyle', k) for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}}
+        return {k: TheConfig.getValue('PointStyle', k) for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}}
     elif role == SplitLineStyleRole:
       if all([k in self.pointStyles[index.row()] for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}]):
         return {k: self.pointStyles[index.row()][k] for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}}
@@ -117,13 +128,13 @@ class WptModel(QAbstractTableModel):
         return {k: TheConfig.getValue('PointStyle', k) for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}}
     elif role == Qt.BackgroundRole:
       if not self.includeStates[index.row()]:
-        return SkipColor
+        return self.SkipColor
       elif self.captionStates[index.row()]:
-        return CaptionColor
+        return self.CaptionColor
       elif self.markerStates[index.row()]:
-        return MarkerColor
+        return self.MarkerColor
       else:
-        return Qt.white
+        return Qt.transparent
     elif role == Qt.FontRole:
       font = QFont()
       if self.splitStates[index.row()]:
@@ -160,10 +171,11 @@ class WptModel(QAbstractTableModel):
       return super(WptModel, self).parent()
 
   def copyToClipboard(self, IDs):
-    text = ''
-    for i in IDs:
-      text += '\t'.join([self.index(i, f).data() for f in TheConfig.columnsToCopy]) + '\n'
-    QGuiApplication.clipboard().setText(text)
+    if len(IDs) > 0:
+      text = ''
+      for i in IDs:
+        text += '\t'.join([self.index(i, f).data() for f in TheConfig.columnsToCopy]) + '\n'
+      QGuiApplication.clipboard().setText(text)
 
   def getSkippedPoints(self):
     return [i for i, s in enumerate(self.includeStates) if not s]
@@ -183,7 +195,7 @@ class WptModel(QAbstractTableModel):
   def getPointStyles(self, key):
     if key in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}:
       return [p[key] for i, p in enumerate(self.pointStyles) if self.markerStates[i]]
-    if key in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}:
+    if key in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}:
       return [p[key] for i, p in enumerate(self.pointStyles) if self.captionStates[i]]
     if key in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}:
       return [p[key] for i, p in enumerate(self.pointStyles) if self.splitStates[i]]
@@ -218,7 +230,7 @@ class WptModel(QAbstractTableModel):
   def setCaptionStates(self, IDs, state):
     for i in IDs:
       self.captionStates[i] = state
-      for key in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}:
+      for key in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}:
         if key not in self.pointStyles[i]:
           self.pointStyles[i][key] = TheConfig.getValue('PointStyle', key)
     self.wptDataChanged.emit()
@@ -231,10 +243,11 @@ class WptModel(QAbstractTableModel):
           self.pointStyles[i][key] = TheConfig.getValue('PointStyle', key)
     self.wptDataChanged.emit()
 
-  def setNeglectStates(self, IDs, state):
+  def setNeglectStates(self, IDs, state, update=True):
     for i in IDs:
       self.neglectStates[i] = state
-    self.parent().updateDistance()
+    if update:
+      self.parent().updateDistance()
     self.wptDataChanged.emit()
 
   def setPointStyle(self, IDs, key, value):
@@ -258,6 +271,10 @@ class TrkModel(QAbstractTableModel):
     self.fields = [self.tr('Name'), self.tr('Segments'), self.tr('Points'), self.tr('Length'), self.tr('Time'), self.tr('Duration')]
     self.resetModel()
 
+    # Define colors that should look well for any color theme
+    lightness = max(50, min(240, QGuiApplication.palette().base().color().lightness()))
+    self.SkipColor = QColor.fromHsl(0, lightness, lightness)
+
   def rowCount(self, parent=None):
     if parent is not None and parent.isValid():
       return 0
@@ -279,7 +296,7 @@ class TrkModel(QAbstractTableModel):
     elif role == IncludeRole:
       return self.includeStates[index.row()]
     elif role == Qt.BackgroundRole:
-      return Qt.white if self.includeStates[index.row()] else SkipColor
+      return QGuiApplication.palette().base().color() if self.includeStates[index.row()] else self.SkipColor
     return None
 
   def headerData(self, section, orientation, role):
@@ -326,8 +343,8 @@ class GpxParser(QObject):
   def parse(self, filename):
     try:
       doc = ET.parse(filename)
-    except ET.ParseError:
-      raise GpxWarning(filename + self.tr(' is an invalid GPX file.'))
+    except (ET.ParseError, IsADirectoryError, FileNotFoundError):
+      raise GpxWarning(QFileInfo(filename).absoluteFilePath() + self.tr(' is an invalid GPX file.'))
     ns = {'ns': doc.getroot().tag.split('}')[0][1:]}
 
     tag = 'name'
@@ -336,7 +353,6 @@ class GpxParser(QObject):
     elif TheConfig.getValue('ProfileStyle', 'ReadNameFromTag') == 2:  # Description
       tag = 'desc'
 
-    wptid = self.wptmodel.rowCount()
     for p in doc.iterfind('.//{%(ns)s}wpt' % ns):
       try:
         point = {}
@@ -344,17 +360,31 @@ class GpxParser(QObject):
         point[NAME] = name.strip() if name is not None else ''
         point[LAT] = float(p.get('lat'))
         point[LON] = float(p.get('lon'))
-        point[ALT] = int(round(float(p.findtext('{%(ns)s}ele' % ns))))
+        ele = p.findtext('{%(ns)s}ele' % ns)
+        point[ALT] = float(ele) if ele is not None else 0
         self.minalt = min(self.minalt, point[ALT])
         self.maxalt = max(self.maxalt, point[ALT])
-        if p.findtext('{%(ns)s}time' % ns) is not None:
-          dt = datetime.strptime(p.findtext('{%(ns)s}time' % ns).strip(), '%Y-%m-%dT%H:%M:%SZ')
-          point[TIME] = dt
+        time = p.findtext('{%(ns)s}time' % ns)
+        if time is not None:
+          point[TIME] = datetime.strptime(time[:19].strip(), '%Y-%m-%dT%H:%M:%S')
         else:
           point[TIME] = ''
         point[DIST] = ''
-        point[TIMEDELTA] = ''
+        point[TIME_DELTA] = ''
         point[TIME_DAYS] = ''
+        point[DIST_DELTA] = ''
+        point[ALT_DELTA] = ''
+        point[SPEED] = ''
+        point[ALT_SPEED] = ''
+        point[SLOPE] = ''
+
+        wptid = self.wptmodel.rowCount()
+        # Sort points by time
+        if TheConfig.getValue('ProfileStyle', 'SortByTime') and point[TIME] != '':
+          while wptid > 0 and self.wptmodel.waypoints[wptid - 1][TIME] != '' and point[TIME] < self.wptmodel.waypoints[wptid - 1][TIME]:
+            wptid -= 1
+          for i in range(wptid, self.wptmodel.rowCount()):
+            self.wptmodel.waypoints[i]['ID'] += 1
         point['ID'] = wptid
 
         # Additional fields
@@ -363,15 +393,14 @@ class GpxParser(QObject):
         point['SYM'] = p.findtext('{%(ns)s}sym' % ns)
 
         self.wptmodel.beginInsertRows(QModelIndex(), wptid, wptid)
-        self.wptmodel.waypoints += [point]
-        self.wptmodel.includeStates += [True]
-        self.wptmodel.markerStates += [False]
-        self.wptmodel.captionStates += [False]
-        self.wptmodel.splitStates += [False]
-        self.wptmodel.neglectStates += [False]
-        self.wptmodel.pointStyles += [{}]
+        self.wptmodel.waypoints.insert(wptid, point)
+        self.wptmodel.includeStates.insert(wptid, True)
+        self.wptmodel.markerStates.insert(wptid, False)
+        self.wptmodel.captionStates.insert(wptid, False)
+        self.wptmodel.splitStates.insert(wptid, False)
+        self.wptmodel.neglectStates.insert(wptid, False)
+        self.wptmodel.pointStyles.insert(wptid, {})
         self.wptmodel.endInsertRows()
-        wptid += 1
 
       except (TypeError, ValueError):
         self.warningSent.emit(self.tr('File read error'),
@@ -396,12 +425,13 @@ class GpxParser(QObject):
             point = {}
             point[LAT] = float(p.get('lat'))
             point[LON] = float(p.get('lon'))
-            point[ALT] = int(round(float(p.findtext('{%(ns)s}ele' % ns))))
+            ele = p.findtext('{%(ns)s}ele' % ns)
+            point[ALT] = float(ele) if ele is not None else 0
             self.minalt = min(self.minalt, point[ALT])
             self.maxalt = max(self.maxalt, point[ALT])
-            if p.findtext('{%(ns)s}time' % ns) is not None:
-              dt = datetime.strptime(p.findtext('{%(ns)s}time' % ns).strip(), '%Y-%m-%dT%H:%M:%SZ')
-              point[TIME] = dt
+            time = p.findtext('{%(ns)s}time' % ns)
+            if time is not None:
+              point[TIME] = datetime.strptime(time[:19].strip(), '%Y-%m-%dT%H:%M:%S')
             else:
               point[TIME] = ''
 
@@ -434,8 +464,8 @@ class GpxParser(QObject):
         self.warningSent.emit(self.tr('File read error'),
                               self.tr('Track ') + (track[TRKNAME] + ' ' if track[TRKNAME] != '' else '') + self.tr('is invalid and will be skipped.', 'Track'))
 
-    TheConfig['ProfileStyle']['MinimumAltitude'] = str(round(self.minalt, -3) - 500 if round(self.minalt, -3) > self.minalt else round(self.minalt, -3))
-    TheConfig['ProfileStyle']['MaximumAltitude'] = str(round(self.maxalt, -3) + 500 if round(self.maxalt, -3) < self.maxalt else round(self.maxalt, -3))
+    TheConfig['ProfileStyle']['MinimumAltitude'] = str(int(round(self.minalt, -3) - 500 if round(self.minalt, -3) > self.minalt else round(self.minalt, -3)))
+    TheConfig['ProfileStyle']['MaximumAltitude'] = str(int(round(self.maxalt, -3) + 500 if round(self.maxalt, -3) < self.maxalt else round(self.maxalt, -3)))
 
   def updatePoints(self):
     self.points = []
@@ -465,8 +495,8 @@ class GpxParser(QObject):
         if self.wptmodel.includeStates[i]:
           self.points += [i]
 
-    self.updateDistance()
     self.updateTimeDifference()
+    self.updateDistance()
 
   def updateDistance(self):
     prev_lat = None
@@ -480,7 +510,7 @@ class GpxParser(QObject):
         lon = self.wptmodel.waypoints[p][LON]
         if not self.wptmodel.neglectStates[p] and prev_lat is not None and prev_lon is not None:
           dist += _distance(lat, lon, prev_lat, prev_lon) * dist_coeff
-        self.wptmodel.waypoints[p][DIST] = round(dist, 3)
+        self.wptmodel.waypoints[p][DIST] = dist
         prev_lat = lat
         prev_lon = lon
       else:
@@ -488,15 +518,17 @@ class GpxParser(QObject):
         lon = self.trkmodel.tracks[p[0]]['SEGMENTS'][p[1]][p[2]][LON]
         if prev_lat is not None and prev_lon is not None:
           dist += _distance(lat, lon, prev_lat, prev_lon) * dist_coeff
-        self.trkmodel.tracks[p[0]]['SEGMENTS'][p[1]][p[2]][DIST] = round(dist, 3)
+        self.trkmodel.tracks[p[0]]['SEGMENTS'][p[1]][p[2]][DIST] = dist
         prev_lat = lat
         prev_lon = lon
 
     for i in self.wptmodel.getSkippedPoints():
       self.wptmodel.waypoints[i][DIST] = ''
 
-    for i in self.trkmodel.tracks:
-      i[TRKLEN] = round(i['LENGTH'] * dist_coeff, 3)
+    for t in self.trkmodel.tracks:
+      t[TRKLEN] = round(t['LENGTH'] * dist_coeff, 3)
+
+    self.updateDetailedData()
 
   def updateTimeDifference(self):
     start_dt = None
@@ -511,10 +543,10 @@ class GpxParser(QObject):
 
     for i, p in enumerate(self.wptmodel.waypoints):
       if self.wptmodel.includeStates[i] and p[TIME] != '':
-        p[TIMEDELTA] = p[TIME] - start_dt
-        p[TIME_DAYS] = round(p[TIMEDELTA].days + p[TIMEDELTA].seconds / 60.0 / 60.0 / 24.0, 3)
+        p[TIME_DELTA] = p[TIME] - start_dt
+        p[TIME_DAYS] = p[TIME_DELTA].days + p[TIME_DELTA].seconds / 60.0 / 60.0 / 24.0
       else:
-        p[TIMEDELTA] = ''
+        p[TIME_DELTA] = ''
         p[TIME_DAYS] = ''
 
     for i, track in enumerate(self.trkmodel.tracks):
@@ -523,10 +555,48 @@ class GpxParser(QObject):
           for p in seg:
             if p[TIME] != '':
               time_delta = p[TIME] - start_dt
-              p[TIME_DAYS] = round(time_delta.days + time_delta.seconds / 60.0 / 60.0 / 24.0, 3)
+              p[TIME_DAYS] = time_delta.days + time_delta.seconds / 60.0 / 60.0 / 24.0
             else:
               p[TIME_DAYS] = ''
 
+  def updateDetailedData(self):
+    if self.wptmodel.rowCount() == 0:
+      return
+
+    i = 0
+    for i, p in enumerate(self.wptmodel.waypoints):
+      if self.wptmodel.includeStates[i]:
+        prev = p
+        break
+      else:
+        p[DIST_DELTA] = ''
+        p[ALT_DELTA] = ''
+        p[SPEED] = ''
+        p[ALT_SPEED] = ''
+        p[SLOPE] = ''
+
+    prev[DIST_DELTA] = 0.0
+    prev[ALT_DELTA] = 0
+    prev[SPEED] = 0.0
+    prev[ALT_SPEED] = 0.0
+    prev[SLOPE] = 0.0
+
+    for j, p in enumerate(self.wptmodel.waypoints[i+1:], i + 1):
+      if self.wptmodel.includeStates[j]:
+        dt = (p[TIME_DAYS] - prev[TIME_DAYS]) * 24.0 if p[TIME_DAYS] != '' and prev[TIME_DAYS] != 0 else 0
+        p[DIST_DELTA] = p[DIST] - prev[DIST]
+        p[ALT_DELTA] = p[ALT] - prev[ALT]
+        p[SPEED] = p[DIST_DELTA] / dt if dt != 0 else 0.0
+        p[ALT_SPEED] = p[ALT_DELTA] / dt if dt != 0 else 0.0
+        p[SLOPE] = p[ALT_DELTA] / p[DIST_DELTA] if p[DIST_DELTA] != 0 else 0.0
+        prev = p
+      else:
+        p[DIST_DELTA] = ''
+        p[ALT_DELTA] = ''
+        p[SPEED] = ''
+        p[ALT_SPEED] = ''
+        p[SLOPE] = ''
+
   def writeToFile(self, filename):
     root = ET.Element('gpx', attrib={'version': '1.1',
                                      'creator': 'GPXViewer - https://bitbucket.org/salsergey/gpxviewer',
@@ -553,9 +623,10 @@ class GpxParser(QObject):
         el = ET.Element('ele')
         el.text = str(p[ALT])
         element.append(el)
-        el = ET.Element('time')
-        el.text = p[TIME].strftime('%Y-%m-%dT%H:%M:%SZ')
-        element.append(el)
+        if p[TIME] != '':
+          el = ET.Element('time')
+          el.text = p[TIME].strftime('%Y-%m-%dT%H:%M:%SZ')
+          element.append(el)
         el = ET.Element('name')
         if p['ID'] in self.wptmodel.changedNames:
           el.text = self.wptmodel.changedNames[p['ID']]
@@ -662,6 +733,12 @@ class GpxSortFilterModel(QSortFilterProxyModel):
     else:
       return False
 
+  def filterAcceptsColumn(self, source_column, source_parent):
+    if TheConfig.getboolean('MainWindow', 'DetailedView'):
+      return True
+    else:
+      return source_column < TheConfig.defaultColumnsNumber
+
   def setFilterMask(self, skipped, marked, captioned, markedCaptioned, other):
     self.includeSkipped = skipped
     self.includeMarked = marked
@@ -738,6 +815,34 @@ def markerPath(style, size):
     path.lineTo(center + r * QPointF(cos(2*pi/3), sin(2*pi/3)))
     path.moveTo(center)
     path.lineTo(center + r * QPointF(cos(4*pi/3), sin(4*pi/3)))
+  elif style == 'ad':
+    path.moveTo(center + r * QPointF(0, 1))
+    path.lineTo(center + 0.7 * r * QPointF(-1, 0))
+    path.moveTo(center + r * QPointF(0, 1))
+    path.lineTo(center + 0.7 * r * QPointF(1, 0))
+    path.moveTo(center + r * QPointF(0, 1))
+    path.lineTo(center + r * QPointF(0, -1))
+  elif style == 'au':
+    path.moveTo(center + r * QPointF(0, -1))
+    path.lineTo(center + 0.7 * r * QPointF(-1, 0))
+    path.moveTo(center + r * QPointF(0, -1))
+    path.lineTo(center + 0.7 * r * QPointF(1, 0))
+    path.moveTo(center + r * QPointF(0, -1))
+    path.lineTo(center + r * QPointF(0, 1))
+  elif style == 'al':
+    path.moveTo(center + r * QPointF(-1, 0))
+    path.lineTo(center + 0.7 * r * QPointF(0, -1))
+    path.moveTo(center + r * QPointF(-1, 0))
+    path.lineTo(center + 0.7 * r * QPointF(0, 1))
+    path.moveTo(center + r * QPointF(-1, 0))
+    path.lineTo(center + r * QPointF(1, 0))
+  elif style == 'ar':
+    path.moveTo(center + r * QPointF(1, 0))
+    path.lineTo(center + 0.7 * r * QPointF(0, -1))
+    path.moveTo(center + r * QPointF(1, 0))
+    path.lineTo(center + 0.7 * r * QPointF(0, 1))
+    path.moveTo(center + r * QPointF(1, 0))
+    path.lineTo(center + r * QPointF(-1, 0))
   elif style == 's':
     path.addRect((1 - sqrt(0.5)) * r, (1 - sqrt(0.5)) * r, sqrt(2.0) * r, sqrt(2.0) * r)
   elif style == 'p':
@@ -804,7 +909,7 @@ def _markerIcon(style, color):
   pix.fill(Qt.transparent)
   p = QPainter(pix)
   p.setRenderHint(QPainter.Antialiasing)
-  if style in {'1', '2', '3', '4', '+', 'x', '_', '|'}:
+  if style in {'1', '2', '3', '4', 'ad', 'au', 'al', 'ar', '+', 'x', '_', '|'}:
     p.setPen(QPen(QColor(color), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
   else:
     p.setPen(QPen(QColor(color), 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
