@@ -1,6 +1,6 @@
 # gpxviewer
 #
-# Copyright (C) 2016-2019 Sergey Salnikov <salsergey@gmail.com>
+# Copyright (C) 2016-2020 Sergey Salnikov <salsergey@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3
@@ -15,12 +15,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import webbrowser
-from math import ceil
-from PyQt5.QtCore import Qt, QCoreApplication, QEvent, QMargins, QSortFilterProxyModel, pyqtSignal, pyqtSlot
-from PyQt5.QtGui import qAlpha, QColor, QCursor, QFont, QGuiApplication, QIcon, QPalette, QPen
-from PyQt5.QtWidgets import QAction, QDialog, QMenu, QMessageBox, QSizePolicy
-from qcustomplot import (QCP, QCustomPlot, QCPAxisTickerFixed, QCPDataRange, QCPDataSelection,
-                         QCPGraph, QCPItemPosition, QCPItemText, QCPScatterStyle)
+from datetime import timedelta
+from PyQt5.QtCore import Qt, QCoreApplication, QDate, QDateTime, QFileSelector, QMargins, pyqtSignal, pyqtSlot
+from PyQt5.QtGui import QColor, QCursor, QFont, QGuiApplication, QIcon, QPen
+from PyQt5.QtWidgets import QAction, QDialog, QMenu, QMessageBox
+from QCustomPlot2 import (QCP, QCustomPlot, QCPAxisTickerDateTime, QCPAxisTickerFixed, QCPDataRange, QCPDataSelection,
+                          QCPGraph, QCPItemPosition, QCPItemText, QCPItemTracer, QCPScatterStyle, QCPTextElement)
 import gpxviewer.gpxmodel as gpx
 from gpxviewer.configstore import TheConfig
 from gpxviewer.gpxdocument import TheDocument
@@ -30,31 +30,49 @@ from gpxviewer.pointconfigdialog import PointConfigDialog
 class PlotCanvas(QCustomPlot):
   def __init__(self, parent=None):
     super(QCustomPlot, self).__init__(parent)
-    self.font = QFont()
-    self.font.setStyleHint(QFont.SansSerif)
 
     self.axisRect().setupFullAxesBox(True)
+    self.xAxis.setTickLabelPadding(10)
+    self.yAxis.setTickLabelPadding(10)
+    self.xAxis.grid().setVisible(False)
+
     self.xTicker = AxisTicker(Qt.Horizontal)
     self.xAxis.setTicker(self.xTicker)
-    self.xTicker2 = AxisTicker(Qt.Horizontal)
-    self.xAxis2.setTicker(self.xTicker2)
+    self.xAxis2.setTicker(self.xTicker)
     self.yTicker = AxisTicker(Qt.Vertical)
     self.yAxis.setTicker(self.yTicker)
-    self.yTicker2 = AxisTicker(Qt.Vertical)
-    self.yAxis2.setTicker(self.yTicker2)
-
-    self.xAxis.grid().setVisible(False)
-    gridColor = QColor(Qt.gray)
-    gridColor.setAlpha(150)
-    self.yAxis.grid().setPen(QPen(gridColor, 1, Qt.DashLine))
+    self.yAxis2.setTicker(self.yTicker)
 
     self.addLayer('profile', self.layer('main'), QCustomPlot.limBelow)
 
     self.setInteraction(QCP.iSelectItems)
     self.setInteraction(QCP.iSelectPlottables)
+    self.setInteraction(QCP.iRangeDrag)
+    self.setInteraction(QCP.iRangeZoom)
+    self.axisRect().setRangeDrag(Qt.Horizontal)
+    self.axisRect().setRangeZoom(Qt.Horizontal)
 
     self.selectionChangedByUser.connect(self.onSelectionChanged)
 
+    self.font = QFont()
+    self.font.setStyleHint(QFont.SansSerif)
+
+    self.themeSelector = QFileSelector()
+    self.themeSelector.setExtraSelectors([TheConfig['MainWindow']['ColorTheme']])
+
+    self.showInfo = False
+    self.setAutoAddPlottableToLegend(False)
+    self.xLegendText = QCPTextElement(self)
+    self.yLegendText = QCPTextElement(self)
+    self.xLegendText.setMinimumSize(200, 0)
+    self.yLegendText.setMinimumSize(200, 0)
+    self.xLegendText.setLayer(self.legend.layer())
+    self.yLegendText.setLayer(self.legend.layer())
+    self.legend.addElement(self.xLegendText)
+    self.legend.addElement(self.yLegendText)
+    self.xLegendText.setMargins(QMargins(10, 5, 10, 5))
+    self.yLegendText.setMargins(QMargins(10, 5, 10, 5))
+
   def deselectAll(self):
     super(PlotCanvas, self).deselectAll()
     self.onSelectionChanged()
@@ -77,36 +95,55 @@ class PlotCanvas(QCustomPlot):
     self.xx = []
     self.yy = []
     self.markers = []
+    self.infoMarkers = []
     self.captions = []
     self.splitLines = []
-    self.neglectPoints = [0]
+    self.neglectPoints = []
+
+  def fitWidth(self):
+    self.xAxis.setRange(self.xx[0], self.xx[-1])
+    self.replot()
+
+  def updatePoints(self):
+    if TheConfig.getValue('ProfileStyle', 'SelectedPointsOnly') \
+       and TheConfig.getValue('ProfileStyle', 'StartFromZero') \
+       and self.column in {gpx.DIST, gpx.TIME_DAYS} and len(self.wptRows) != 0:
+      startDist = float(TheDocument.wptmodel.index(self.wptRows[0], self.column).data())
+    else:
+      startDist = 0
 
-  def updatePoints(self, wptRows, trkRows):
-    zeroDist = 0
     for p in TheDocument.gpxparser.points:
       if type(p) == int:  # points
-        if (p in wptRows or len(wptRows) == 0) and (self.column == gpx.DIST or TheDocument.wptmodel.index(p, self.column).data() != ''):
-          self.xx += [float(TheDocument.wptmodel.index(p, self.column).data()) - zeroDist]
+        if (p in self.wptRows or len(self.wptRows) == 0) \
+           and (self.column == gpx.DIST or TheDocument.wptmodel.index(p, self.column).data() != ''):
+          if self.column in {gpx.DIST, gpx.TIME_DAYS}:
+            self.xx += [float(TheDocument.wptmodel.index(p, self.column).data()) - startDist]
+          else:  # absolute time
+            self.xx += [QCPAxisTickerDateTime.dateTimeToKey(QDateTime.fromString(
+                        TheDocument.wptmodel.index(p, self.column).data(), 'yyyy-MM-dd HH:mm:ss'))]
           self.yy += [float(TheDocument.wptmodel.index(p, gpx.ALT).data())]
-          if len(self.xx) == 1:
-            zeroDist = self.xx[0]
-            self.xx[0] = 0
-          if p != 0 and TheDocument.wptmodel.index(p, 0).data(gpx.NeglectRole):
+          if p != 0 and self.column == gpx.DIST and TheDocument.wptmodel.index(p, 0).data(gpx.NeglectRole):
             self.neglectPoints += [self.xx[-1]]
           if TheDocument.wptmodel.index(p, 0).data(gpx.SplitLineRole):
             self.splitLines += [SplitLine(self, p, self.xx[-1], self.yy[-1])]
           if TheDocument.wptmodel.index(p, 0).data(gpx.MarkerRole):
             self.markers += [MarkerItem(self, p, self.xx[-1], self.yy[-1])]
+          else:
+            self.infoMarkers += [MarkerItem(self, p, self.xx[-1], self.yy[-1])]
+            self.infoMarkers[-1].setVisible(False)
           if TheDocument.wptmodel.index(p, 0).data(gpx.CaptionRole):
             self.captions += [CaptionItem(self, p, self.xx[-1], self.yy[-1])]
-      elif p[0] in trkRows or len(trkRows) == 0 and len(wptRows) == 0:  # tracks
+      elif p[0] in self.trkRows or len(self.trkRows) == 0 and len(self.wptRows) == 0:  # tracks
         if self.column == gpx.DIST or TheDocument.trkmodel.index(p[0], gpx.TRKTIME).data() != '':
-          self.xx += [float(TheDocument.trkmodel.tracks[p[0]]['SEGMENTS'][p[1]][p[2]][self.column]) - zeroDist]
+          if self.column in {gpx.DIST, gpx.TIME_DAYS}:
+            self.xx += [float(TheDocument.trkmodel.tracks[p[0]]['SEGMENTS'][p[1]][p[2]][self.column]) - startDist]
+          else:  # absolute time
+            # Convert track time to a proper timezone
+            self.xx += [QCPAxisTickerDateTime.dateTimeToKey(QDateTime.fromString(
+                        str(TheDocument.trkmodel.tracks[p[0]]['SEGMENTS'][p[1]][p[2]][self.column]
+                            + timedelta(minutes=TheConfig.getValue('ProfileStyle', 'TimeZoneOffset'))), 'yyyy-MM-dd HH:mm:ss'))]
           self.yy += [float(TheDocument.trkmodel.tracks[p[0]]['SEGMENTS'][p[1]][p[2]][gpx.ALT])]
-          if len(self.xx) == 1:
-            zeroDist = self.xx[0]
-            self.xx[0] = 0
-    self.neglectPoints += [self.xx[-1]]
+    self.neglectPoints = [self.xx[0]] + self.neglectPoints + [self.xx[-1]]
 
     if TheConfig.getValue('ProfileStyle', 'AutoscaleAltitudes'):
       minalt, maxalt = min(self.yy), max(self.yy)
@@ -117,7 +154,29 @@ class PlotCanvas(QCustomPlot):
       self.maxalt = TheConfig.getValue('ProfileStyle', 'MaximumAltitude')
 
   def updateAxes(self):
-    self.xAxis.setRangeUpper(self.xx[-1])
+    if TheConfig.getValue('ProfileStyle', 'UseSystemTheme'):
+      textColor = QGuiApplication.palette().text().color()
+      self.setBackground(QGuiApplication.palette().base())
+      self.legend.setBorderPen(textColor)
+      self.legend.setBrush(QGuiApplication.palette().base())
+      gridColor = QGuiApplication.palette().light().color() if TheConfig['MainWindow']['ColorTheme'] == 'dark_theme' \
+             else QGuiApplication.palette().mid().color()
+    else:
+      textColor = QColor(Qt.black)
+      self.setBackground(QColor(Qt.white))
+      self.legend.setBorderPen(textColor)
+      self.legend.setBrush(QColor(Qt.white))
+      gridColor = QColor(Qt.gray)
+
+    for axis in {self.xAxis, self.xAxis2, self.yAxis, self.yAxis2}:
+      axis.setBasePen(QPen(textColor))
+      axis.setTickPen(QPen(textColor))
+      axis.setSubTickPen(QPen(textColor))
+      axis.setLabelColor(textColor)
+      axis.setTickLabelColor(textColor)
+    gridColor.setAlpha(150)
+    self.yAxis.grid().setPen(QPen(gridColor, 1, Qt.DashLine))
+
     self.yAxis.setRange(self.minalt, self.maxalt)
     self.xTicker.setType(self.column)
 
@@ -128,25 +187,38 @@ class PlotCanvas(QCustomPlot):
     self.yAxis.setTickLabelFont(self.font)
     self.yAxis.setLabelFont(self.font)
 
+    for text in {self.xLegendText, self.yLegendText}:
+      text.setFont(self.font)
+      text.setTextColor(textColor)
+
+    if self.column == gpx.TIME:
+      self.xAxis.setTickLabelRotation(-90)
+    else:
+      self.xAxis.setTickLabelRotation(0)
+
     if self.column == gpx.DIST:
       dist_coeff = TheConfig.getValue('ProfileStyle', 'DistanceCoefficient')
-      if dist_coeff == 1.0:
-        self.xAxis.setLabel(self.tr('Distance (km)'))
-      else:
+      if dist_coeff != 1.0 and TheConfig.getValue('ProfileStyle', 'ShowDistanceCoefficient'):
         self.xAxis.setLabel(self.tr('Distance with coefficient ') + str(dist_coeff) + self.tr(' (km)'))
+      else:
+        self.xAxis.setLabel(self.tr('Distance (km)'))
     else:  # time
-      if self.xx[-1] > 1:
+      if self.xx[-1] - self.xx[0] > 1:
         self.xAxis.setLabel(self.tr('Time (days)'))
       else:
         self.xAxis.setLabel(self.tr('Time (hours)'))
     self.yAxis.setLabel(self.tr('Altitude (m)'))
 
-  def plotProfile(self, column, wptRows, trkRows):
+  def plotProfile(self, column, wptRows, trkRows, fit=True):
     self.column = column
+    self.wptRows = wptRows
+    self.trkRows = trkRows
     self.reset()
     self.setCurrentLayer('main')
-    self.updatePoints(wptRows, trkRows)
+    self.updatePoints()
     self.updateAxes()
+    if fit:
+      self.fitWidth()
 
     self.setCurrentLayer('profile')
     for n in range(1, len(self.neglectPoints)):
@@ -158,6 +230,9 @@ class PlotCanvas(QCustomPlot):
                            self.yy[self.xx.index(self.neglectPoints[n-1]) + (1 if n > 1 else 0):self.xx.index(self.neglectPoints[n]) + 1])
       self.graph().setSelectable(False)
 
+    self.setCurrentLayer('main')
+    if self.showInfo:
+      self.showInformation(True)
     self.replot()
 
   def saveProfile(self, filename, figsize=None):
@@ -175,6 +250,22 @@ class PlotCanvas(QCustomPlot):
       QMessageBox.warning(self, self.tr('Save error'), self.tr('Error writing file ') + filename +
                           self.tr('.\nProbably the given format isn\'t supported by the system.'))
 
+  def showInformation(self, enable):
+    if enable:
+      self.tracer = QCPItemTracer(self)
+      self.tracer.setSelectable(False)
+      lineColor = QGuiApplication.palette().text().color() if TheConfig.getValue('ProfileStyle', 'UseSystemTheme') else QColor(Qt.black)
+      self.tracer.setPen(QPen(lineColor))
+
+    self.showInfo = enable
+    self.tracer.setVisible(enable)
+    self.legend.setVisible(enable)
+    for m in self.infoMarkers:
+      m.setVisible(enable)
+      m.setSelectable(enable)
+    self.deselectAll()
+    self.replot()
+
   def keyPressEvent(self, event):
     if event.key() == Qt.Key_Escape:
       if self.selectedElement is not None:
@@ -182,8 +273,10 @@ class PlotCanvas(QCustomPlot):
         self.replot()
       else:
         super(PlotCanvas, self).keyPressEvent(event)
+
     elif event.key() == Qt.Key_Menu and self.selectedElement is not None:
       self.contextMenu()
+
     elif event.key() in {Qt.Key_Left, Qt.Key_Right, Qt.Key_Down, Qt.Key_Up} and len(self.selectedItems()) == 1:
       x = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionStyleRole)[gpx.CAPTION_POSX]
       y = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionStyleRole)[gpx.CAPTION_POSY]
@@ -196,13 +289,52 @@ class PlotCanvas(QCustomPlot):
       elif event.key() == Qt.Key_Up:
         TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSY, y + 1)
       self.replot()
+
     else:
       super(PlotCanvas, self).keyPressEvent(event)
 
+  def mouseMoveEvent(self, event):
+    if self.axisRect().rect().contains(event.pos()):
+      # Show information
+      if self.showInfo:
+        if self.plottableAt(event.pos(), True) is None and self.itemAt(event.pos(), True) is None:
+          self.setCursor(QCursor(Qt.CrossCursor))
+        else:
+          self.setCursor(QCursor())
+
+        for profile in self.layer('profile').children():
+          if profile.getKeyRange()[0].contains(self.xAxis.pixelToCoord(event.pos().x())):
+            self.tracer.setGraph(profile)
+        self.tracer.setGraphKey(self.xAxis.pixelToCoord(event.pos().x()))
+        self.replot()
+        if self.column == gpx.DIST:
+          self.xLegendText.setText(self.tr('Distance: ') + str(round(self.tracer.position.key(), 3)))
+        elif self.column == gpx.TIME_DAYS:
+          self.xLegendText.setText(self.tr('Time: ') + str(round(self.tracer.position.key(), 3)))
+        else:  # absolute time
+          self.xLegendText.setText(self.tr('Time: ') +
+                                   QCPAxisTickerDateTime.keyToDateTime(self.tracer.position.key()).toString('yyyy-MM-dd HH:mm:ss'))
+        self.yLegendText.setText(self.tr('Altitude: ') + str(round(self.tracer.position.value())))
+        self.replot()
+
+      else:  # not showing information
+        if self.plottableAt(event.pos(), True) is None and self.itemAt(event.pos(), True) is None:
+          self.setCursor(QCursor(Qt.OpenHandCursor))
+        else:
+          self.setCursor(QCursor())
+
+    # Outside the axes rect
+    elif event.pos().x() < self.axisRect().rect().left():
+      self.setCursor(QCursor(Qt.SizeVerCursor))
+    else:
+      self.setCursor(QCursor())
+
+    super(PlotCanvas, self).mouseMoveEvent(event)
+
   def mouseReleaseEvent(self, event):
     if event.button() == Qt.RightButton:
       self.deselectAll()
-      item = self.itemAt(event.pos())
+      item = self.itemAt(event.pos(), True)
       if item is None:
         item = self.plottableAt(event.pos(), True)
       if item is not None:
@@ -210,6 +342,7 @@ class PlotCanvas(QCustomPlot):
         self.onSelectionChanged()
         self.contextMenu()
       self.replot()
+
     super(PlotCanvas, self).mouseReleaseEvent(event)
 
   def wheelEvent(self, event):
@@ -224,22 +357,28 @@ class PlotCanvas(QCustomPlot):
       self.updateAxes()
       self.replot()
       self.profileChanged.emit()
-    super(PlotCanvas, self).wheelEvent(event)
+
+    else:  # inside the axes rect
+      super(PlotCanvas, self).wheelEvent(event)
 
   def contextMenu(self):
-    actMarker = QAction(QIcon(':/icons/waypoint-marker.svg'), self.tr('Points with markers'), self)
+    actMarker = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-marker.svg')), self.tr('Point with marker'), self)
     actMarker.setCheckable(True)
     actMarker.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.MarkerRole))
     actMarker.triggered.connect(self.onMarkerPoints)
-    actCaption = QAction(QIcon(':/icons/waypoint-caption.svg'), self.tr('Points with captions'), self)
+    actCaption = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-caption.svg')), self.tr('Point with caption'), self)
     actCaption.setCheckable(True)
     actCaption.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionRole))
     actCaption.triggered.connect(self.onCaptionPoints)
-    actSplit = QAction(QIcon(':/icons/waypoint-splitline.svg'), self.tr('Points with splitting lines'), self)
+    actSplit = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-splitline.svg')), self.tr('Point with splitting line'), self)
     actSplit.setCheckable(True)
     actSplit.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.SplitLineRole))
     actSplit.triggered.connect(self.onSplitLines)
-    actStyle = QAction(QIcon(':/icons/configure.svg'), self.tr('Point style'), self)
+    actSkip = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-skip.svg')), self.tr('Skip point'), self)
+    actSkip.setCheckable(True)
+    actSkip.setChecked(not TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.IncludeRole))
+    actSkip.triggered.connect(self.onSkipPoints)
+    actStyle = QAction(QIcon(self.themeSelector.select(':/icons/configure.svg')), self.tr('Point style'), self)
     actStyle.triggered.connect(self.onPointStyle)
 
     menu = QMenu(self)
@@ -247,6 +386,8 @@ class PlotCanvas(QCustomPlot):
     menu.addAction(actCaption)
     menu.addAction(actSplit)
     menu.addSeparator()
+    menu.addAction(actSkip)
+    menu.addSeparator()
     menu.addAction(actStyle)
     menu.addSeparator()
 
@@ -264,7 +405,7 @@ class PlotCanvas(QCustomPlot):
     actShowTopoMap.triggered.connect(self.showTopoMap)
 
     showMapMenu = QMenu(self.tr('Show on map'), self)
-    showMapMenu.setIcon(QIcon(':/icons/internet-services.svg'))
+    showMapMenu.setIcon(QIcon(self.themeSelector.select(':/icons/internet-services.svg')))
     showMapMenu.addAction(actShowGoogleMap)
     showMapMenu.addAction(actShowYandexMap)
     showMapMenu.addAction(actShowZoomEarthMap)
@@ -279,13 +420,15 @@ class PlotCanvas(QCustomPlot):
     checked = self.sender().isChecked()
     TheDocument.wptmodel.setMarkerStates([self.selectedElement.idx], checked)
     if checked:
-      self.markers += [MarkerItem(self, self.selectedElement.idx, self.selectedElement.posX, self.selectedElement.posY)]
+      index = [m.idx for m in self.infoMarkers].index(self.selectedElement.idx)
+      self.markers += [self.infoMarkers.pop(index)]
     else:
-      for i, m in enumerate(self.markers):
-        if m.idx == self.selectedElement.idx:
-          self.removeGraph(m)
-          del self.markers[i]
-          break
+      index = [m.idx for m in self.markers].index(self.selectedElement.idx)
+      self.infoMarkers += [self.markers.pop(index)]
+      if not self.showInfo:
+        self.infoMarkers[-1].setVisible(False)
+        self.infoMarkers[-1].setSelectable(False)
+
     self.replot()
 
   @pyqtSlot()
@@ -316,6 +459,12 @@ class PlotCanvas(QCustomPlot):
           break
     self.replot()
 
+  @pyqtSlot()
+  def onSkipPoints(self):
+    checked = self.sender().isChecked()
+    TheDocument.wptmodel.setIncludeStates([self.selectedElement.idx], not checked)
+    self.plotProfile(self.column, self.wptRows, self.trkRows, False)
+
   def onPointStyle(self):
     dlg = PointConfigDialog(self, self.selectedElement.idx)
     if dlg.exec_() == QDialog.Accepted:
@@ -377,47 +526,66 @@ class AxisTicker(QCPAxisTickerFixed):
 
   def setType(self, type):
     self.type = type
+    if self.type == gpx.TIME:
+      self.setTickOrigin(QCPAxisTickerDateTime.dateTimeToKey(QDate(2019, 1, 1)))
+    else:
+      self.setTickOrigin(0)
 
   def getTickStep(self, range):
     if self.orientation == Qt.Horizontal:
       if self.type == gpx.DIST:
-        if range.upper > 200:
-          return 20
-        elif range.upper > 100:
-          return 10
-        elif range.upper > 50:
-          return 5
-        elif range.upper > 20:
-          return 2
-        else:
-          return 1
+        availableSteps = [0.1, 0.2, 0.5, 1, 2, 5, 10, 20, 50, 100]
       else:  # time
-        if range.upper > 1:  # days
-          return 1
-        else:  # hours
-          return 1.0 / 24
+        availableSteps = [1.0 / 24, 2.0 / 24, 3.0 / 24, 6.0 / 24, 12.0 / 24, 1, 2]
+        if self.type == gpx.TIME:
+          availableSteps = [s * 24 * 3600 for s in availableSteps]  # convert to seconds
+      return self.pickClosest(range.size() / 10, availableSteps)
     else:  # vertical
-      if range.size() >= 2000:
-        return 500
-      elif range.size() >= 1000:
-        return 200
-      else:
-        return 100
+      availableSteps = [10, 20, 50, 100, 200, 500, 1000]
+      return self.pickClosest(range.size() / 7, availableSteps)
 
   def getSubTickCount(self, tickStep):
     if self.orientation == Qt.Horizontal:
-      if self.type == gpx.DIST and tickStep % 2 == 0:
-        return 1
-      else:
-        return 0
+      if self.type == gpx.DIST:
+        if tickStep in {0.1, 0.5, 1, 5, 10, 50, 100}:
+          return 4
+        else:
+          return 1
+      else:  # time
+        if tickStep in {12.0 / 24, 12.0 * 3600}:
+          return 3
+        elif tickStep in {3.0 / 24, 3.0 * 3600}:
+          return 2
+        elif tickStep in {2.0 / 24, 6.0 / 24, 2, 2.0 * 3600, 6.0 * 3600, 2 * 24 * 3600}:
+          return 1
+        else:
+          return 0
     else:  # vertical
-      return int(tickStep / 100) - 1
+      if tickStep in {10, 50, 100, 500, 1000}:
+        return 4
+      else:
+        return 1
 
   def createLabelVector(self, ticks, locale, formatChar, precision):
-    if self.orientation == Qt.Horizontal and self.type == gpx.TIME_DAYS and ticks[-1] <= 1:  # hours
-      return [str(int(round(24 * t))) for t in ticks]
-    else:
-      return super(AxisTicker, self).createLabelVector(ticks, locale, formatChar, precision)
+    if self.orientation == Qt.Horizontal:
+      if self.type == gpx.TIME_DAYS:
+        if ticks[-1] < 1:  # hours
+          return [str(int(round(24 * t))) for t in ticks]
+        elif any([t % 1.0 != 0 for t in ticks]):  # days and hours
+          if TheConfig.getValue('ProfileStyle', 'ShowHours'):
+            return [QCoreApplication.translate('PlotCanvas', 'day ') + str(int(t)) + "\n"
+                    + str(int(round(24 * t)) % 24).zfill(2) + ":00" for t in ticks]
+          else:
+            return [str(int(t)) if t % 1.0 == 0 else "" for t in ticks]
+      elif self.type == gpx.TIME:
+        if any([QCPAxisTickerDateTime.keyToDateTime(t).toString('HH') != '00' for t in ticks]) \
+           and TheConfig.getValue('ProfileStyle', 'ShowHours'):  # days and hours
+          return [QCPAxisTickerDateTime.keyToDateTime(t).toString('d MMM\nHH:mm') for t in ticks]
+        else:
+          return [QCPAxisTickerDateTime.keyToDateTime(t).toString('d MMM')
+                  if QCPAxisTickerDateTime.keyToDateTime(t).toString('HH') == '00' else "" for t in ticks]
+
+    return super(AxisTicker, self).createLabelVector(ticks, locale, formatChar, precision)
 
 
 class MarkerItem(QCPGraph):
@@ -436,19 +604,31 @@ class MarkerItem(QCPGraph):
     super(MarkerItem, self).draw(painter)
 
   def updateStyle(self):
-    markerStyle = TheDocument.wptmodel.index(self.idx, gpx.NAME).data(gpx.MarkerStyleRole)
     scatterStyle = QCPScatterStyle()
-    scatterStyle.setCustomPath(gpx.markerPath(markerStyle[gpx.MARKER_STYLE], 2 * markerStyle[gpx.MARKER_SIZE]))  # for better compatibility with matplotlib
-    scatterStyle.setBrush(QColor.fromRgba(markerStyle[gpx.MARKER_COLOR]))
-    if markerStyle[gpx.MARKER_STYLE] in {'1', '2', '3', '4', '+', 'x', '_', '|'}:
-      scatterStyle.setPen(QPen(QColor.fromRgba(markerStyle[gpx.MARKER_COLOR]), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+    if self.idx in TheDocument.wptmodel.getMarkedPoints():
+      markerStyle = TheDocument.wptmodel.index(self.idx, gpx.NAME).data(gpx.MarkerStyleRole)
+      scatterStyle.setCustomPath(gpx.markerPath(markerStyle[gpx.MARKER_STYLE], 2 * markerStyle[gpx.MARKER_SIZE]))  # for better compatibility with matplotlib
+      scatterStyle.setBrush(QColor.fromRgba(markerStyle[gpx.MARKER_COLOR]))
+      if markerStyle[gpx.MARKER_STYLE] in {'1', '2', '3', '4', 'ad', 'au', 'al', 'ar', '+', 'x', '_', '|'}:
+        scatterStyle.setPen(QPen(QColor.fromRgba(markerStyle[gpx.MARKER_COLOR]), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+      else:
+        scatterStyle.setPen(QPen(QColor.fromRgba(markerStyle[gpx.MARKER_COLOR]), 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
     else:
-      scatterStyle.setPen(QPen(QColor.fromRgba(markerStyle[gpx.MARKER_COLOR]), 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
-    self.setScatterStyle(scatterStyle)
+      scatterStyle.setShape(QCPScatterStyle.ssDisc)
+      scatterStyle.setSize(3 * TheConfig.getValue('ProfileStyle', 'ProfileWidth'))
+      scatterStyle.setBrush(QColor.fromRgba(TheConfig.getValue('ProfileStyle', 'ProfileColor')))
+      scatterStyle.setPen(QPen(QColor.fromRgba(TheConfig.getValue('ProfileStyle', 'ProfileColor')), 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
 
     selectedScatterStyle = QCPScatterStyle()
-    selectedScatterStyle.setCustomPath(gpx.markerPath(markerStyle[gpx.MARKER_STYLE], 3 * markerStyle[gpx.MARKER_SIZE]))
-    selectedScatterStyle.setPen(QPen(QGuiApplication.palette().color(QPalette.Highlight), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+    if self.idx in TheDocument.wptmodel.getMarkedPoints():
+      selectedScatterStyle.setCustomPath(gpx.markerPath(markerStyle[gpx.MARKER_STYLE], 3 * markerStyle[gpx.MARKER_SIZE]))
+      selectedScatterStyle.setPen(QPen(QGuiApplication.palette().highlight().color(), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+    else:
+      selectedScatterStyle.setShape(QCPScatterStyle.ssCircle)
+      selectedScatterStyle.setSize(6 * TheConfig.getValue('ProfileStyle', 'ProfileWidth'))
+      selectedScatterStyle.setPen(QPen(QGuiApplication.palette().highlight().color(), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+
+    self.setScatterStyle(scatterStyle)
     self.selectionDecorator().setScatterStyle(selectedScatterStyle)
     self.selectionDecorator().setUsedScatterProperties(QCPScatterStyle.ScatterProperty(QCPScatterStyle.spPen | QCPScatterStyle.spShape))
 
@@ -463,17 +643,23 @@ class CaptionItem(QCPItemText):
     self.font.setFamily(TheConfig.getValue('ProfileStyle', 'FontFamily'))
     self.setText(TheDocument.wptmodel.index(self.idx, gpx.NAME).data())
 
+    if TheConfig.getValue('ProfileStyle', 'UseSystemTheme'):
+      self.setColor(QGuiApplication.palette().text().color())
+      self.setSelectedColor(QGuiApplication.palette().highlightedText().color())
+    else:
+      self.setColor(QColor(Qt.black))
+      self.setSelectedColor(QColor(Qt.black))
+
     self.setClipToAxisRect(False)
     self.setPadding(QMargins(3, 1, 3, 1))
-    self.setRotation(-90)
+    self.setRotation(-TheDocument.wptmodel.index(self.idx, gpx.NAME).data(gpx.CaptionStyleRole)[gpx.CAPTION_ROTATION])
     self.setPositionAlignment(Qt.AlignLeft | Qt.AlignVCenter)
     self.position.setType(QCPItemPosition.ptPlotCoords)
 
-    fillColor = QGuiApplication.palette().color(QPalette.Highlight)
-    fillColor.setAlpha(100)
+    fillColor = QGuiApplication.palette().highlight().color()
+    fillColor.setAlpha(150)
     self.setSelectedBrush(fillColor)
-    self.setSelectedPen(QGuiApplication.palette().color(QPalette.Highlight))
-    self.setSelectedColor(QColor(Qt.black))
+    self.setSelectedPen(QGuiApplication.palette().highlight().color())
 
   def draw(self, painter):
     self.update()
